==> nvshmem_src_2.7.0-6.txz: nvshmem_src_2.7.0-6/src/coll/host/alltoall.h <==
/*
 * Copyright (c) 2020, NVIDIA CORPORATION. All rights reserved.
 *
 * See COPYRIGHT for license information
 */

#ifndef NVSHMEMI_ALLTOALL_COMMON_CPU_H
#define NVSHMEMI_ALLTOALL_COMMON_CPU_H
#include <cuda.h>
#include <cuda_runtime.h>

template <typename TYPE>
void nvshmemi_call_alltoall_on_stream_kernel(nvshmem_team_t team, TYPE *dest, const TYPE *source,
                                             size_t nelems, cudaStream_t stream);

template<typename TYPE>
int nvshmemi_alltoall_on_stream(                                                  
    nvshmem_team_t team, TYPE *dest, const TYPE *source, size_t nelems, cudaStream_t stream) { 
#ifdef NVSHMEM_USE_NCCL
    nvshmemi_team_t *teami = nvshmemi_team_pool[team];                                         
    int team_n_pes = nvshmem_team_n_pes(team);                                                 
    if (nvshmemi_use_nccl && nvshmemi_get_nccl_dt<TYPE>() != ncclNumTypes &&                                       
        ((nccl_version >= 2700 && team_n_pes <= 4096 /* NCCL limit for Group API */) ||        
         (nccl_version >= 2800 && team_n_pes <= 32768 /* NCCL limit for Group API */))) {      
        size_t rank_offset = nelems * sizeof(TYPE);                                            
        NCCL_CHECK(nccl_ftable.GroupStart());                                                  
        for (int pe = 0; pe < team_n_pes; pe++) {                                              
            NCCL_CHECK(nccl_ftable.Send(((char *)source) + pe * rank_offset, nelems,           
                                        nvshmemi_get_nccl_dt<TYPE>(), pe,                
                                        teami->nccl_comm, stream));                            
            NCCL_CHECK(nccl_ftable.Recv(((char *)dest) + pe * rank_offset, nelems,             
                                        nvshmemi_get_nccl_dt<TYPE>(), pe,                
                                        teami->nccl_comm, stream));                            
        }                                                                                      
        NCCL_CHECK(nccl_ftable.GroupEnd());                                                    
    } else
#endif /* NVSHMEM_USE_NCCL */
    {                                                                                   
        nvshmemi_call_alltoall_on_stream_kernel<TYPE>(team, dest, source, nelems, stream);     
    }                                                                                          
    return 0;                                                                                  
}

#endif /* NVSHMEMI_ALLTOALL_COMMON_CPU_H */

==> nvshmem_src_2.8.0-3.txz: nvshmem_src_2.8.0-3/src/coll/host/alltoall.h <==
/*
 * Copyright (c) 2020, NVIDIA CORPORATION. All rights reserved.
 *
 * See COPYRIGHT for license information
 */

#ifndef NVSHMEMI_ALLTOALL_COMMON_CPU_H
#define NVSHMEMI_ALLTOALL_COMMON_CPU_H
#include <cuda.h>
#include <cuda_runtime.h>

template <typename TYPE>
void nvshmemi_call_alltoall_on_stream_kernel(nvshmem_team_t team, TYPE *dest, const TYPE *source,
                                             size_t nelems, cudaStream_t stream);

template <typename TYPE>
int nvshmemi_alltoall_on_stream(nvshmem_team_t team, TYPE *dest, const TYPE *source, size_t nelems,
                                cudaStream_t stream) {
#ifdef NVSHMEM_USE_NCCL
    nvshmemi_team_t *teami = nvshmemi_team_pool[team];
    int team_n_pes = nvshmem_team_n_pes(team);
    if (nvshmemi_use_nccl && nvshmemi_get_nccl_dt<TYPE>() != ncclNumTypes &&
        ((nccl_version >= 2700 && team_n_pes <= 4096 /* NCCL limit for Group API */) ||
         (nccl_version >= 2800 && team_n_pes <= 32768 /* NCCL limit for Group API */))) {
        size_t rank_offset = nelems * sizeof(TYPE);
        NCCL_CHECK(nccl_ftable.GroupStart());
        for (int pe = 0; pe < team_n_pes; pe++) {
            NCCL_CHECK(nccl_ftable.Send(((char *)source) + pe * rank_offset, nelems,
                                        nvshmemi_get_nccl_dt<TYPE>(), pe, teami->nccl_comm,
                                        stream));
            NCCL_CHECK(nccl_ftable.Recv(((char *)dest) + pe * rank_offset, nelems,
                                        nvshmemi_get_nccl_dt<TYPE>(), pe, teami->nccl_comm,
                                        stream));
        }
        NCCL_CHECK(nccl_ftable.GroupEnd());
    } else
#endif /* NVSHMEM_USE_NCCL */
    {
        nvshmemi_call_alltoall_on_stream_kernel<TYPE>(team, dest, source, nelems, stream);
    }
    return 0;
}

#endif /* NVSHMEMI_ALLTOALL_COMMON_CPU_H */
